==> resh-5.3.0.6-py3-none-any.whl: resh/domains/module.py <==
import os
import threading
from time import sleep

from resh.domains._domain import Domain
from resh.protocols.can_utils.can_packet import CanDump
from resh.protocols.can_utils.interface import CanConnection
from resh.protocols.can_utils.rapida import commands as rcommands, responses as rresp
from resh.protocols.can_utils.rapida_packet import RapidaSend, RapidaDump
from resh.utils import groups, monitor_filter, tools, compat, settings
from resh.utils.const import *
from resh.utils.errors import CustomError, DeviceNotAnswered, ModuleConstructError, BadParameter, PacketParseError, \
    CanOpenError, NegativeResult
from resh.utils.logger import log

"""Loader"""
BL_INFO = 0xB0       # запрос информации о загрузчике
LOAD_INIT = 0xB2     # инициация загрузки

DALI_TYPE = 1
MODBUS_TYPE = 2

# Время для загрузки прошивки при перезапуске модуля, сек
REBOOT_DELAY = 5


@compat.add_metaclass(compat.DomainTracker)
class Module(Domain):
    def_params = {MODULE: 2}
    domain_name = MODULE_DOMAIN
    _module_type = 1

    def _send_raw(self, frame: bytes):
        if not isinstance(frame, (bytes, bytearray)):
            try:
                frame = bytes([int(b) for b in frame])
            except:
                raise BadParameter()

        message = RapidaSend(self._module_type, self.module, frame)
        return message.send_raw()

    def _send_cmd(self, cmd, *params, filters=None, **kwargs) -> rresp.Response:
        if cmd == rcommands.Reboot:
            kwargs['answer_timeout'] = kwargs.get('answer_timeout', REBOOT_DELAY)

        try:
            _cmd = cmd(*[int(p) for p in params])
        except:
            raise BadParameter()

        log.info(f'-> {_cmd}')
        pack = RapidaSend(self._module_type, self.module, _cmd.frame, **kwargs)
        if filters is not None:
            pack.bus.set_filters(filters=filters)

        return pack.send(_cmd.is_query)

    def _send(self, frame=None, need_answer=False, **kwargs):
        try:
            if frame is None:
                frame = []
                while self.has_next:
                    frame.append(int(self.next_param))

            elif isinstance(frame, int):
                frame = [frame]
            else:
                frame = list(frame)

            frame = bytes(frame)
        except:
            raise BadParameter()

        message = RapidaSend(self._module_type, self.module, frame, **kwargs)
        return message.send(need_answer=need_answer)

    def _listen_for(self, cmd_byte, timeout=0.05):
        message = RapidaSend(self._module_type, self.module, [cmd_byte-1], answer_timeout=timeout)
        message.start_answer_timer()
        return message.receive()


"""https://kiwi.throne.systems/index.php/Tron/Strata/Hardware/Rapida/Resh/Command/Rapida#Base"""


class raw_command(Module, groups.Base):
    def_params = {CLASS: 1, MODULE_0_31: 2}
    opt_params = {BYTES_ARR: '1 0xff 255'}

    def exec(self):
        try:
            self._module_type = int(self.next_param)
        except:
            raise BadParameter()
        self._send()


class raw_query(Module, groups.Base):
    def_params = {CLASS: 1, MODULE_0_31: 2}
    opt_params = {BYTES_ARR: '1 0xff 255'}

    def exec(self):
        try:
            self._module_type = int(self.next_param)
        except:
            raise BadParameter()
        return self._send(need_answer=True)


"""https://kiwi.throne.systems/index.php/Tron/Strata/Hardware/Rapida/Resh/Command/Rapida#Standard"""


class readdress(Module, groups.Common):
    def_params = {}
    var_params = {FORCED: ''}

    def exec(self):
        self._module = 0
        try:
            self._send_cmd(rcommands.Reboot, answer_timeout=0.5)
        except DeviceNotAnswered:
            pass

        addressed = []
        forced = False

        if FORCED in self._args:
            forced = True
            self._args.remove(FORCED)

        try:
            self._send_cmd(rcommands.CommitAddresses, answer_timeout=5)
        except DeviceNotAnswered as ex:
            if forced:
                try:
                    self._send_cmd(rcommands.RunAutoAddressing, answer_timeout=5)
                except DeviceNotAnswered:
                    pass
                finally:
                    try:
                        self._send_cmd(rcommands.CommitAddresses)
                    except DeviceNotAnswered:
                        pass
            else:
                stack = ex.stack
                if stack is None or 0 not in stack:
                    raise ModuleConstructError('Expected answer from 01F is absent!')
                if len(stack) > 1:
                    stack.pop(0)
                    raise ModuleConstructError('Unexpected answer from modules {}'.format(stack.keys()))
                stack = stack.pop(0)
                unaddressed = []
                for dump in stack:
                    if isinstance(dump.payload.command, rresp.UnaddressedNotify):
                        if dump.data not in unaddressed:
                            unaddressed.append(dump.data)
                if len(unaddressed) == 0:
                    raise ModuleConstructError('no module has responded to 0xAD')

                expect_count = len(unaddressed)

                try:
                    self._send_cmd(rcommands.RunAutoAddressing, filters=RapidaSend.filters(0x3e0), answer_timeout=5)
                except DeviceNotAnswered as ex:
                    stack = ex.stack
                    if stack is None:
                        stack = {}
                    if RapidaSend.CONTROLLER_ID not in stack:
                        print('RunAutoAddressing(0xAA) command not answered')
                    else:
                        for dump in stack[RapidaSend.CONTROLLER_ID]:
                            if isinstance(dump.payload.command, rcommands.NextAddress):
                                addressed.append(dump.data[0] - 1)
                        if len(addressed) == 0:
                            print('No module answered on 0xAA')
                        elif addressed.count(1) > 1:
                            print('One of the not outer slots is empty')
                        elif expect_count != len(addressed):
                            print('{} modules found but {} addressed. Wtf?'.format(expect_count, len(addressed)))
                finally:
                    try:
                        self._send_cmd(rcommands.CommitAddresses)
                    except DeviceNotAnswered:
                        pass

                return len(addressed)


class find(Module, groups.Common):
    def_params = {}

    def exec(self):
        self._module = 0
        res = []
        try:
            self._send_cmd(rcommands.QueryUidMin, filters=[])
        except DeviceNotAnswered as ex:
            if ex.stack:
                for m_id, dumps in ex.stack.copy().items():
                    for dump in dumps:
                        if isinstance(dump, RapidaDump):
                            res.append(
                                dict(address=m_id,
                                     type=dump.module_type,
                                     name=dump.module_name,
                                     desc=dump.module_desc)
                            )
                            break
        return sorted(res, key=lambda item: item['address'])


class set_address(Module, groups.Common):
    opt_params = {'new_address': 4}

    def exec(self):
        new_address = self.next_param
        self._send_cmd(rcommands.SetAddress, new_address)
        self._module = new_address
        self._listen_for(0x01, 5)


class info(Module, groups.Common):
    def exec(self):
        res = version(*self._args).exec()
        res['serial'] = serial(*self._args).exec()
        return res


class _update_firmware(Module, groups.Common):
    TIMEOUT = 40
    opt_params = {FILE: 'path/to/file.ext'}
    _stopped = False
    _can = None
    _exit_timer = None
    _exit_by_timeout = False

    """
    Пакеты, получаемые после обновления прошивки:
    BLFWReceived, BLFWSuccess, BootLoaderVersion, FirmwareVersion
    """

    def exec(self):
        path = self.next_param
        if not os.path.exists(path):
            raise CustomError('File "{}" not exist!'.format(path))
        if not os.path.isfile(path):
            raise CustomError('"{}" is not a file!'.format(path))

        try:
            # start_quiescent_mode
            self._send_raw(bytes.fromhex('0102FFFFFE1D'))
            sleep(1)

            try:
                self._send_cmd(rcommands.Reboot, answer_timeout=0.05)
            except DeviceNotAnswered:
                pass

            self._send_cmd(rcommands.InitLoad, answer_timeout=5)
            self.exit_timer.start()
        except DeviceNotAnswered as ex:
            raise ex
        else:
            self._can = CanConnection(interface=settings.conf.interface)
            self._can.set_filter(self.module)

            threading.Thread(target=self.upload, args=[path]).start()
            result = None

            try:
                for dump in self._can.bus:
                    dump = CanDump.can_dump(dump)
                    if isinstance(dump, RapidaDump):
                        try:
                            payload = dump.payload
                            log.info('   <- %s' % payload.command)
                            if isinstance(payload.command, rresp.BLFWSuccess):
                                result = True
                            elif isinstance(payload.command, rresp.FirmwareVersion):
                                if result is None:
                                    raise NegativeResult('Модуль перезагружен, но прошивка не установлена')
                                elif isinstance(result, NegativeResult):
                                    raise result
                                else:
                                    return payload.command.value
                            elif isinstance(payload.command, (rresp.BLFWError, rresp.RebootReason)):
                                result = NegativeResult(str(payload.command))
                        except NegativeResult as ex:
                            raise ex
                        except BaseException as ex:
                            continue
            except NegativeResult as ex:
                raise ex
            except:
                pass
        finally:
            self.do_exit()

            # stop_quiescent_mode
            self._send_raw(bytes.fromhex('0102FFFFFE1E'))

            if self._exit_by_timeout:
                raise NegativeResult('Модуль не ответил в течение %s сек' % self.TIMEOUT)

    @property
    def exit_timer(self):
        if self._exit_timer is None:
            self._exit_timer = threading.Timer(self.TIMEOUT, function=self.on_timeout)

        return self._exit_timer

    def on_timeout(self):
        self._exit_timer = None
        self._exit_by_timeout = True
        self.do_exit()

    def do_exit(self):
        self._stopped = True
        if self._can:
            self._can.stop()
            self._can = None

        if self._exit_timer:
            self._exit_timer.cancel()

    def upload(self, path):
        with open(path, 'rb') as file:
            file.seek(0, 2)
            log.info('uploading %skB ...' % round(file.tell() / 1024))
            file.seek(0, 0)
            while not self._stopped:
                chunk = file.read(8)
                if chunk:
                    self._send_raw(chunk)
                    continue
                else:
                    log.info('uploading success')
                    return

            log.info('uploading break')


class update_firmware(_update_firmware):
    pass


class update_power_firmware(_update_firmware):
    def_params = {}
    _module_type = 31
    _module = 30


class reboot(Module, groups.Common):
    def exec(self): self._send_cmd(rcommands.Reboot)


class version(Module, groups.Common):
    def exec(self):
        val = self._send_cmd(rcommands.QueryVersion).value
        return dict(hardwareVersion=val['hv'],
                    firmwareVersion=val['fv'],
                    bootloaderVersion=val['bv'])


class serial(Module, groups.Common):
    def exec(self):
        _serial = self._send_cmd(rcommands.QueryUidMax).data
        _serial += self._send_cmd(rcommands.QueryUidMin).data
        return ''.join(hex(b)[2:].rjust(2, '0').upper() for b in _serial)


""" Dali """


class __Dali(Module, groups.Dali):
    _module_type = DALI_TYPE


class dali_status(__Dali):
    def exec(self):
        r = self._send_cmd(rcommands.QueryLineStatus)
        r = bin(r.data[0])[:1:-1].ljust(8, '0')
        result = [dict(id=i + 1, powered=r[i] == '1') for i in range(0, len(r))]
        return result


""" Modbus """


class __Modbus(Module, groups.Modbus):
    def_params = {MODULE: 1, CHANNEL: 2}
    _module_type = MODBUS_TYPE


class modbus_channel_status(__Modbus):
    def exec(self): return self._send_cmd(rcommands.QueryChannelConfig, self.channel).value


class modbus_reset_channel(__Modbus):
    def exec(self): return self._send_cmd(rcommands.ResetChannel, self.channel)


class modbus_set_channel_conf(__Modbus):
    opt_params = {BAUD: 2400, PARITY: 'odd', STOP_BITS: 2}

    def exec(self):
        try:
            baud = rresp.ChannelConfig._values["baud"].index(self.next_param)
            parity = rresp.ChannelConfig._values["parity"].index(self.next_param)
            stop = rresp.ChannelConfig._values["stop"].index(self.next_param)
        except:
            raise BadParameter()

        return self._send_cmd(rcommands.SetChannelConfig, self.channel, baud, parity, stop)


""" Wizards """


class monitor(Module, groups.Wizards):
    def_params = {}
    var_params = {MODULE: 1, CHANNEL: 1, DEVICE_ADDR: 'D3,-D4', INSTANCE: 'IT4', NAME: 'event'}

    @classmethod
    def example(cls):
        return ' '.join([cls.domain_name, cls.__name__] +
                        [f'{cls.def_params[p]}' for p in cls.def_params] +
                        [f'{cls.opt_params[p]}' for p in cls.opt_params] +
                        [f'{cls.var_params[p]}' for p in cls.var_params]).lower() + \
               '\n' + cls.loc(cls.domain_name, cls.__name__, NAMED_PARAMS_DESC) + \
               '\n' + cls.loc(cls.domain_name, cls.__name__, EXCLUDE_PARAMS_DESC) + \
               '\n' + cls.loc(cls.domain_name, cls.__name__, BROADCAST_VALUE)

    def _prepare(self):
        self.filter = monitor_filter.Filter(self._args)
        self.can = CanConnection(interface=settings.conf.interface)

    def exec(self):
        try:
            self._prepare()
        except OSError as ex:
            raise CanOpenError(reason=str(ex))

        try:
            for dump in self.can.bus:
                try:
                    dump = CanDump.can_dump(dump)
                    if dump is None:
                        continue
                    payload = dump.payload
                    if self.filter.check(payload):
                        log.monitor(payload)
                except BaseException as ex:
                    try:
                        log.exception(PacketParseError(dump.payload.header_str() + ' ' + str(ex)))
                    except:
                        log.exception(PacketParseError(tools.bytes2str(dump)))
        except:
            return

    def set_break(self, sig, params):
        super(monitor, self).set_break(sig, params)
        self.can.stop()

==> resh-5.4.0.10-py3-none-any.whl: resh/domains/module.py <==
import os
import threading
from time import sleep

from resh.domains._domain import Domain
# from resh.protocols.can_utils.can_packet import CanDump
# from resh.protocols.can_utils.interface import CanConnection
from resh.protocols.can_utils.rapida import commands as rcommands, responses as rresp
# from resh.protocols.can_utils.rapida_packet import RapidaSend, RapidaDump
from resh.utils import groups, monitor_filter, tools, compat, settings
from resh.utils.const import *
from resh.utils.errors import CustomError, DeviceNotAnswered, ModuleConstructError, BadParameter, PacketParseError, \
    CanOpenError, NegativeResult
from resh.utils.logger import log

"""Loader"""
BL_INFO = 0xB0       # запрос информации о загрузчике
LOAD_INIT = 0xB2     # инициация загрузки

DALI_TYPE = 1
MODBUS_TYPE = 2

# Время для загрузки прошивки при перезапуске модуля, сек
REBOOT_DELAY = 5


@compat.add_metaclass(compat.DomainTracker)
class Module(Domain):
    def_params = {MODULE: 2}
    domain_name = MODULE_DOMAIN
    _module_type = 1

    def __init__(self, *args):
        super(Module, self).__init__(*args)
        global RapidaSend
        from resh.protocols.can_utils.rapida_packet import RapidaSend

    def _send_raw(self, frame: bytes):
        if not isinstance(frame, (bytes, bytearray)):
            try:
                frame = bytes([int(b) for b in frame])
            except:
                raise BadParameter()

        message = RapidaSend(self._module_type, self.module, frame)
        return message.send_raw()

    def _send_cmd(self, cmd, *params, filters=None, **kwargs) -> rresp.Response:
        if cmd == rcommands.Reboot:
            kwargs['answer_timeout'] = kwargs.get('answer_timeout', REBOOT_DELAY)

        try:
            _cmd = cmd(*[int(p) for p in params])
        except:
            raise BadParameter()

        log.info(f'-> {_cmd}')
        pack = RapidaSend(self._module_type, self.module, _cmd.frame, **kwargs)
        if filters is not None:
            pack.bus.set_filters(filters=filters)

        return pack.send(_cmd.is_query)

    def _send(self, frame=None, need_answer=False, **kwargs):
        try:
            if frame is None:
                frame = []
                while self.has_next:
                    frame.append(int(self.next_param))

            elif isinstance(frame, int):
                frame = [frame]
            else:
                frame = list(frame)

            frame = bytes(frame)
        except:
            raise BadParameter()

        message = RapidaSend(self._module_type, self.module, frame, **kwargs)
        return message.send(need_answer=need_answer)

    def _listen_for(self, cmd_byte, timeout=0.05):
        message = RapidaSend(self._module_type, self.module, [cmd_byte-1], answer_timeout=timeout)
        message.start_answer_timer()
        return message.receive()


"""https://kiwi.throne.systems/index.php/Tron/Strata/Hardware/Rapida/Resh/Command/Rapida#Base"""


class raw_command(Module, groups.Base):
    def_params = {CLASS: 1, MODULE_0_31: 2}
    opt_params = {BYTES_ARR: '1 0xff 255'}

    def exec(self):
        try:
            self._module_type = int(self.next_param)
        except:
            raise BadParameter()
        self._send()


class raw_query(Module, groups.Base):
    def_params = {CLASS: 1, MODULE_0_31: 2}
    opt_params = {BYTES_ARR: '1 0xff 255'}

    def exec(self):
        try:
            self._module_type = int(self.next_param)
        except:
            raise BadParameter()
        return self._send(need_answer=True)


"""https://kiwi.throne.systems/index.php/Tron/Strata/Hardware/Rapida/Resh/Command/Rapida#Standard"""


class readdress(Module, groups.Common):
    def_params = {}
    var_params = {FORCED: ''}

    def exec(self):
        self._module = 0
        try:
            self._send_cmd(rcommands.Reboot, answer_timeout=0.5)
        except DeviceNotAnswered:
            pass

        addressed = []
        forced = False

        if FORCED in self._args:
            forced = True
            self._args.remove(FORCED)

        try:
            self._send_cmd(rcommands.CommitAddresses, answer_timeout=5)
        except DeviceNotAnswered as ex:
            if forced:
                try:
                    self._send_cmd(rcommands.RunAutoAddressing, answer_timeout=5)
                except DeviceNotAnswered:
                    pass
                finally:
                    try:
                        self._send_cmd(rcommands.CommitAddresses)
                    except DeviceNotAnswered:
                        pass
            else:
                stack = ex.stack
                if stack is None or 0 not in stack:
                    raise ModuleConstructError('Expected answer from 01F is absent!')
                if len(stack) > 1:
                    stack.pop(0)
                    raise ModuleConstructError('Unexpected answer from modules {}'.format(stack.keys()))
                stack = stack.pop(0)
                unaddressed = []
                for dump in stack:
                    if isinstance(dump.payload.command, rresp.UnaddressedNotify):
                        if dump.data not in unaddressed:
                            unaddressed.append(dump.data)
                if len(unaddressed) == 0:
                    raise ModuleConstructError('no module has responded to 0xAD')

                expect_count = len(unaddressed)

                try:
                    self._send_cmd(rcommands.RunAutoAddressing, filters=RapidaSend.filters(0x3e0), answer_timeout=5)
                except DeviceNotAnswered as ex:
                    stack = ex.stack
                    if stack is None:
                        stack = {}
                    if RapidaSend.CONTROLLER_ID not in stack:
                        print('RunAutoAddressing(0xAA) command not answered')
                    else:
                        for dump in stack[RapidaSend.CONTROLLER_ID]:
                            if isinstance(dump.payload.command, rcommands.NextAddress):
                                addressed.append(dump.data[0] - 1)
                        if len(addressed) == 0:
                            print('No module answered on 0xAA')
                        elif addressed.count(1) > 1:
                            print('One of the not outer slots is empty')
                        elif expect_count != len(addressed):
                            print('{} modules found but {} addressed. Wtf?'.format(expect_count, len(addressed)))
                finally:
                    try:
                        self._send_cmd(rcommands.CommitAddresses)
                    except DeviceNotAnswered:
                        pass

                return len(addressed)


class find(Module, groups.Common):
    def_params = {}

    def __init__(self, *args):
        super(find, self).__init__(*args)
        global RapidaDump
        from resh.protocols.can_utils.rapida_packet import RapidaDump

    def exec(self):
        self._module = 0
        res = []
        try:
            self._send_cmd(rcommands.QueryUidMin, filters=[])
        except DeviceNotAnswered as ex:
            if ex.stack:
                for m_id, dumps in ex.stack.copy().items():
                    for dump in dumps:
                        if isinstance(dump, RapidaDump):
                            res.append(
                                dict(address=m_id,
                                     type=dump.module_type,
                                     name=dump.module_name,
                                     desc=dump.module_desc)
                            )
                            break
        return sorted(res, key=lambda item: item['address'])


class set_address(Module, groups.Common):
    opt_params = {'new_address': 4}

    def exec(self):
        new_address = self.next_param
        self._send_cmd(rcommands.SetAddress, new_address)
        self._module = new_address
        self._listen_for(0x01, 5)


class info(Module, groups.Common):
    def exec(self):
        res = version(*self._args).exec()
        res['serial'] = serial(*self._args).exec()
        return res


class _update_firmware(Module, groups.Common):
    TIMEOUT = 40
    opt_params = {FILE: 'path/to/file.ext'}
    _stopped = False
    _can = None
    _exit_timer = None
    _exit_by_timeout = False

    """
    Пакеты, получаемые после обновления прошивки:
    BLFWReceived, BLFWSuccess, BootLoaderVersion, FirmwareVersion
    """

    def __init__(self, *args):
        super(_update_firmware, self).__init__(*args)
        global CanConnection, CanDump, RapidaDump
        from resh.protocols.can_utils.interface import CanConnection
        from resh.protocols.can_utils.can_packet import CanDump
        from resh.protocols.can_utils.rapida_packet import RapidaDump

    def exec(self):
        path = self.next_param
        if not os.path.exists(path):
            raise CustomError('File "{}" not exist!'.format(path))
        if not os.path.isfile(path):
            raise CustomError('"{}" is not a file!'.format(path))

        try:
            # start_quiescent_mode
            self._send_raw(bytes.fromhex('0102FFFFFE1D'))
            sleep(1)

            try:
                self._send_cmd(rcommands.Reboot, answer_timeout=0.05)
            except DeviceNotAnswered:
                pass

            self._send_cmd(rcommands.InitLoad, answer_timeout=5)
            self.exit_timer.start()
        except DeviceNotAnswered as ex:
            raise ex
        else:
            self._can = CanConnection(interface=settings.conf.interface)
            self._can.set_filter(self.module)

            threading.Thread(target=self.upload, args=[path]).start()
            result = None

            try:
                for dump in self._can.bus:
                    dump = CanDump.can_dump(dump)
                    if isinstance(dump, RapidaDump):
                        try:
                            payload = dump.payload
                            log.info('   <- %s' % payload.command)
                            if isinstance(payload.command, rresp.BLFWSuccess):
                                result = True
                            elif isinstance(payload.command, rresp.FirmwareVersion):
                                if result is None:
                                    raise NegativeResult('Модуль перезагружен, но прошивка не установлена')
                                elif isinstance(result, NegativeResult):
                                    raise result
                                else:
                                    return payload.command.value
                            elif isinstance(payload.command, (rresp.BLFWError, rresp.RebootReason)):
                                result = NegativeResult(str(payload.command))
                        except NegativeResult as ex:
                            raise ex
                        except BaseException as ex:
                            continue
            except NegativeResult as ex:
                raise ex
            except:
                pass
        finally:
            self.do_exit()

            # stop_quiescent_mode
            self._send_raw(bytes.fromhex('0102FFFFFE1E'))

            if self._exit_by_timeout:
                raise NegativeResult('Модуль не ответил в течение %s сек' % self.TIMEOUT)

    @property
    def exit_timer(self):
        if self._exit_timer is None:
            self._exit_timer = threading.Timer(self.TIMEOUT, function=self.on_timeout)

        return self._exit_timer

    def on_timeout(self):
        self._exit_timer = None
        self._exit_by_timeout = True
        self.do_exit()

    def do_exit(self):
        self._stopped = True
        if self._can:
            self._can.stop()
            self._can = None

        if self._exit_timer:
            self._exit_timer.cancel()

    def upload(self, path):
        with open(path, 'rb') as file:
            file.seek(0, 2)
            log.info('uploading %skB ...' % round(file.tell() / 1024))
            file.seek(0, 0)
            while not self._stopped:
                chunk = file.read(8)
                if chunk:
                    self._send_raw(chunk)
                    continue
                else:
                    log.info('uploading success')
                    return

            log.info('uploading break')


class update_firmware(_update_firmware):
    pass


class update_power_firmware(_update_firmware):
    def_params = {}
    _module_type = 31
    _module = 30


class reboot(Module, groups.Common):
    def exec(self): self._send_cmd(rcommands.Reboot)


class version(Module, groups.Common):
    def exec(self):
        val = self._send_cmd(rcommands.QueryVersion).value
        return dict(hardwareVersion=val['hv'],
                    firmwareVersion=val['fv'],
                    bootloaderVersion=val['bv'])


class serial(Module, groups.Common):
    def exec(self):
        _serial = self._send_cmd(rcommands.QueryUidMax).data
        _serial += self._send_cmd(rcommands.QueryUidMin).data
        return ''.join(hex(b)[2:].rjust(2, '0').upper() for b in _serial)


""" Dali """


class __Dali(Module, groups.Dali):
    _module_type = DALI_TYPE


class dali_status(__Dali):
    def exec(self):
        r = self._send_cmd(rcommands.QueryLineStatus)
        r = bin(r.data[0])[:1:-1].ljust(8, '0')
        result = [dict(id=i + 1, powered=r[i] == '1') for i in range(0, len(r))]
        return result


""" Modbus """


class __Modbus(Module, groups.Modbus):
    def_params = {MODULE: 1, CHANNEL: 2}
    _module_type = MODBUS_TYPE


class modbus_channel_status(__Modbus):
    def exec(self): return self._send_cmd(rcommands.QueryChannelConfig, self.channel).value


class modbus_reset_channel(__Modbus):
    def exec(self): return self._send_cmd(rcommands.ResetChannel, self.channel)


class modbus_set_channel_conf(__Modbus):
    opt_params = {BAUD: 2400, PARITY: 'odd', STOP_BITS: 2}

    def exec(self):
        try:
            baud = rresp.ChannelConfig._values["baud"].index(self.next_param)
            parity = rresp.ChannelConfig._values["parity"].index(self.next_param)
            stop = rresp.ChannelConfig._values["stop"].index(self.next_param)
        except:
            raise BadParameter()

        return self._send_cmd(rcommands.SetChannelConfig, self.channel, baud, parity, stop)


""" Wizards """


class monitor(Module, groups.Wizards):
    def_params = {}
    var_params = {MODULE: 1, CHANNEL: 1, DEVICE_ADDR: 'D3,-D4', INSTANCE: 'IT4', NAME: 'event'}

    def __init__(self, *args):
        super(monitor, self).__init__(*args)
        global CanConnection, CanDump
        from resh.protocols.can_utils.interface import CanConnection
        from resh.protocols.can_utils.can_packet import CanDump

    @classmethod
    def example(cls):
        return ' '.join([cls.domain_name, cls.__name__] +
                        [f'{cls.def_params[p]}' for p in cls.def_params] +
                        [f'{cls.opt_params[p]}' for p in cls.opt_params] +
                        [f'{cls.var_params[p]}' for p in cls.var_params]).lower() + \
               '\n' + cls.loc(cls.domain_name, cls.__name__, NAMED_PARAMS_DESC) + \
               '\n' + cls.loc(cls.domain_name, cls.__name__, EXCLUDE_PARAMS_DESC) + \
               '\n' + cls.loc(cls.domain_name, cls.__name__, BROADCAST_VALUE)

    def _prepare(self):
        self.filter = monitor_filter.Filter(self._args)
        self.can = CanConnection(interface=settings.conf.interface)

    def exec(self):
        try:
            self._prepare()
        except OSError as ex:
            raise CanOpenError(reason=str(ex))

        try:
            for dump in self.can.bus:
                try:
                    dump = CanDump.can_dump(dump)
                    if dump is None:
                        continue
                    payload = dump.payload
                    if self.filter.check(payload):
                        log.monitor(payload)
                except BaseException as ex:
                    try:
                        log.exception(PacketParseError(dump.payload.header_str() + ' ' + str(ex)))
                    except:
                        log.exception(PacketParseError(tools.bytes2str(dump)))
        except:
            return

    def set_break(self, sig, params):
        super(monitor, self).set_break(sig, params)
        self.can.stop()
